N = 1;
for (i = 0; i < N; i++) {
// while(true) {
  try { test(); } catch(exc1) {}
}

function test() {
  let data = os.file.readFile("leak.data", 'binary');

  let cloneBuffer = serialize(null);
  cloneBuffer.clonebuffer = data.buffer;
  try {
    let obj = deserialize(cloneBuffer);
    //print(uneval(obj));
  } catch(exc1) {
    print("Exception: " + exc1);
  }
}

gc();
